@@ -17,6 +17,7 @@
 #################################################################
 import sys, os
 import re
+import gzip
 
 
 #################################################################
@@ -32,7 +33,8 @@ class Vcf(object):
     def __init__(self, inputfile):
         ''' open input vcf, read header lines and save
         information as Header object to initialize Vcf object '''
-        self.header = self.parse_header(inputfile)
+        self.inputfile = inputfile
+        self.header = self.parse_header()
     #end def
 
     class Header(object):
@@ -290,24 +292,41 @@ class Vcf(object):
 
     #end class Variant
 
-    def parse_header(self, inputfile):
+    @staticmethod
+    def read_vcf(inputfile):
+        ''' read vcf file, gzipped or ungzipped,
+        return a generator '''
+        if inputfile.endswith('.gz'):
+            with gzip.open(inputfile, 'rb') as fz:
+                for byteline in fz:
+                    yield byteline.decode()
+                #end for
+            #end with
+        else:
+            with open(inputfile) as fi:
+                for line in fi:
+                    yield line
+                #end for
+            #end with
+        #end if
+    #end def
+
+    def parse_header(self):
         ''' read header and save information as Header object '''
         definitions, columns, IDs_genotypes = '', '', ''
-        with open(inputfile) as fi:
-            for line in fi:
-                if line.startswith('#'): # reading a header line
-                    line_strip = line.rstrip()
-                    if line_strip.startswith('##'): # header definition line
-                        definitions += line_strip + '\n'
-                    elif line_strip.startswith('#CHROM'): # header columns line
-                        columns += line_strip + '\n'
-                        IDs_genotypes = line_strip.split('\t')[9:]
-                    #end if
-                else: # finished to read the header
-                    break # exit and close buffer
+        for line in self.read_vcf(self.inputfile):
+            if line.startswith('#'): # reading a header line
+                line_strip = line.rstrip()
+                if line_strip.startswith('##'): # header definition line
+                    definitions += line_strip + '\n'
+                elif line_strip.startswith('#CHROM'): # header columns line
+                    columns += line_strip + '\n'
+                    IDs_genotypes = line_strip.split('\t')[9:]
                 #end if
-            #end for
-        #end with
+            else: # finished to read the header
+                break # exit and close buffer
+            #end if
+        #end for
 
         # Checking header is correct
         if definitions and columns and IDs_genotypes:
@@ -317,22 +336,42 @@ class Vcf(object):
         #end if
     #end def
 
-    def parse_variants(self, inputfile): # generator
+    def parse_variants(self): # generator
         ''' return a generator to variants stored as Variant objects '''
-        with open(inputfile) as fi:
-            for line in fi:
-                if not line.startswith('#'):
-                    line_strip = line.rstrip()
-                    if line_strip:
-                        try:
-                            yield self.Variant(line_strip, self.header.IDs_genotypes)
-                        except Exception:
-                            raise ValueError('\nERROR in variant VCF structure, missing essential columns\n')
-                        #end try
-                    #end if
+        for line in self.read_vcf(self.inputfile):
+            if not line.startswith('#'):
+                line_strip = line.rstrip()
+                if line_strip:
+                    try:
+                        yield self.Variant(line_strip, self.header.IDs_genotypes)
+                    except Exception:
+                        raise ValueError('\nERROR in variant VCF structure, missing essential columns\n')
+                    #end try
                 #end if
-            #end for
-        #end with
+            #end if
+        #end for
+    #end def
+
+    def write_definitions(self, outputfile_obj):
+        ''' write header definitions to outputfile_obj buffer '''
+        outputfile_obj.write(self.header.definitions)
+    #end def
+
+    def write_columns(self, outputfile_obj):
+        ''' write header columns to outputfile_obj buffer,
+        #CHROM ... '''
+        outputfile_obj.write(self.header.columns)
+    #end def
+
+    def write_header(self, outputfile_obj):
+        ''' write header definitions and columns to outputfile_obj buffer '''
+        self.write_definitions(outputfile_obj)
+        self.write_columns(outputfile_obj)
+    #end def
+
+    def write_variant(self, outputfile_obj, parsed_variant):
+        ''' write parsed_variant (Variant object generated by parse_variants) to outputfile_obj buffer '''
+        outputfile_obj.write(parsed_variant.to_string())
     #end def
 
 #end class Vcf
